/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 *  strict-local
 * @format
 */

/* eslint-env worker, serviceworker */
"use strict";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }
  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function() {
    var self = this,
      args = arguments;
    return new Promise(function(resolve, reject) {
      var gen = fn.apply(self, args);
      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }
      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }
      _next(undefined);
    });
  };
}

const stringToBundle = require("./stringToBundle");

const BUNDLE_CACHE_NAME = "__metroBundleCache";

function getBundleFromBrowserCache(_x) {
  return _getBundleFromBrowserCache.apply(this, arguments);
}

function _getBundleFromBrowserCache() {
  _getBundleFromBrowserCache = _asyncToGenerator(function*(bundleReq) {
    const res = yield fetch(bundleReq, {
      // This forces using the browser cache, in which the initial bundle request
      // will have been stored.
      cache: "force-cache"
    });

    if (!res) {
      return null;
    }

    return stringToBundle(yield res.text());
  });
  return _getBundleFromBrowserCache.apply(this, arguments);
}

function getBundleFromCustomCache(_x2, _x3) {
  return _getBundleFromCustomCache.apply(this, arguments);
}
/**
 * Retrieves a bundle from either the custom bundle cache or the browser cache.
 */

function _getBundleFromCustomCache() {
  _getBundleFromCustomCache = _asyncToGenerator(function*(cache, bundleReq) {
    const res = yield cache.match(bundleReq);

    if (!res) {
      return null;
    }

    return yield res.json();
  });
  return _getBundleFromCustomCache.apply(this, arguments);
}

function getBundle(_x4) {
  return _getBundle.apply(this, arguments);
}
/**
 * Stores a bundle in the custom bundle cache.
 */

function _getBundle() {
  _getBundle = _asyncToGenerator(function*(bundleReq) {
    const cache = yield caches.open(BUNDLE_CACHE_NAME);
    const deltaBundle = yield getBundleFromCustomCache(cache, bundleReq);

    if (deltaBundle != null) {
      return deltaBundle;
    }

    return yield getBundleFromBrowserCache(bundleReq);
  });
  return _getBundle.apply(this, arguments);
}

function setBundle(_x5, _x6) {
  return _setBundle.apply(this, arguments);
}

function _setBundle() {
  _setBundle = _asyncToGenerator(function*(bundleReq, bundle) {
    const bundleJson = JSON.stringify(bundle);
    const bundleJsonRes = new Response(bundleJson, {
      status: 200,
      statusText: "OK",
      headers: new Headers({
        "Content-Length": String(bundleJson.length),
        "Content-Type": "application/json",
        Date: new Date().toUTCString()
      })
    });
    const cache = yield caches.open(BUNDLE_CACHE_NAME); // Store the new initial bundle in cache. We don't need to wait for
    // this operation to complete before returning a response.

    yield cache.put(bundleReq, bundleJsonRes);
  });
  return _setBundle.apply(this, arguments);
}

module.exports = {
  getBundle,
  setBundle
};
